from django.template.loader import render_to_string
from django.urls import reverse

portlets = []

def register_portlet(render, order=0, app_label=None, model_name=None,
        show_in_app=False,
        show_in_app_index=False,
        show_in_model_site=True,
        show_in_model_changelist=False,
        show_in_model_change=False,
        show_in_model_add=False,
        show_in_model_history=False,
        extra_css=None,
        extra_js=None):
    portlets.append({
        "render": render,
        "order": order,
        "app_label": app_label,
        "model_name": model_name,
        "show_in_app": show_in_app,
        "show_in_app_index": show_in_app_index,
        "show_in_model_site": show_in_model_site,
        "show_in_model_changelist": show_in_model_changelist,
        "show_in_model_change": show_in_model_change,
        "show_in_model_add": show_in_model_add,
        "show_in_model_history": show_in_model_history,
        "extra_css": extra_css,
        "extra_js": extra_js,
    })

def get_portlets(app_label, model_name, url_name):
    results = []
    for portlet in portlets:
        show_flag = False
        if portlet["app_label"] is None or portlet["app_label"] == app_label:
            if portlet["show_in_app"]:
                show_flag = True
            else:
                if portlet["show_in_app_index"] and url_name == "app_list":
                    show_flag = True
                else:
                    if portlet["model_name"] is None or portlet["model_name"] == model_name:
                        if portlet["show_in_model_site"]:
                            show_flag = True
                        else:
                            if (portlet["show_in_model_changelist"] and url_name == "{}_{}_changelist".format(app_label, model_name)):
                                show_flag = True
                            if (portlet["show_in_model_change"] and url_name == "{}_{}_change".format(app_label, model_name)):
                                show_flag = True
                            if (portlet["show_in_model_add"] and url_name == "{}_{}_add".format(app_label, model_name)):
                                show_flag = True
                            if (portlet["show_in_model_history"] and url_name == "{}_{}_history".format(app_label, model_name)):
                                show_flag = True
        if show_flag:
            results.append(portlet)
    results.sort(key=lambda e: e["order"])
    return results


class Portlet(object):

    def __call__(self, context=None):
        raise NotImplementedError("Please implement portlet's __call__ method...")


class SidebarNavigation(Portlet):

    def __init__(self, title, navigation):
        self.title = title
        self.navigation = navigation

    def __call__(self, context=None):
        return render_to_string("django-admin-app-sidebar/sidebar-navigation.html", {
            "title": self.title,
            "navigation": self.navigation,
            "request": context["request"],
        })



from django.template import Library
from django.template.loader import render_to_string
from django.utils.html import mark_safe
from ..portlets import get_portlets

register = Library()

@register.simple_tag(takes_context=True)
def portlets_render(context):
    request = context.get("request", None)
    if not request:
        return ""
    if "opts" in context:
        opts = context["opts"]
        app_label = opts.app_label
        model_name = opts.model_name
    else:
        if "app_label" in context:
            app_label = context["app_label"]
        else:
            app_label = ""
        if "model_name" in context:
            model_name = context["model_name"]
        else:
            model_name = None
    url_name = request.resolver_match.url_name
    portlet_htmls = []
    extra_css_paths = []
    extra_js_paths = []
    portlets = get_portlets(app_label, model_name, url_name)
    for portlet in portlets:
        for css in (portlet["extra_css"] or []):
            if not css in extra_css_paths:
                extra_css_paths.append(css)
        for js in (portlet["extra_js"] or []):
            if not js in extra_js_paths:
                extra_js_paths(js)
        html = portlet["render"](context)
        portlet_htmls.append(html)
    return render_to_string("django-admin-app-sidebar/sidebar-portlets.html", {
        "portlets": portlet_htmls,
        "extra_css": extra_css_paths,
        "extra_js": extra_js_paths,
        "fix_changelist_problem": url_name.endswith("_changelist") and len(portlets),
        "fix_changeform_problem": (url_name.endswith("_change") or url_name.endswith("_add")) and len(portlets),
    })

@register.simple_tag
def sidebar_navigation_url(item, request):
    url = item["url"]
    url_args = item.get("url_args", [])
    url_kwargs = item.get("url_kwargs", {})
    if callable(url):
        return url(request, *url_args, **url_kwargs)
    else:
        try:
            url = reverse(item.url, args=url_args, kwargs=url_kwargs)
        except:
            return url

